﻿-- Author      : RisM
-- Create Date : 9/21/2009 3:07:03 AM

local SpeakinSpell = LibStub("AceAddon-3.0"):GetAddon("SpeakinSpell")
local L = LibStub("AceLocale-3.0"):GetLocale("SpeakinSpell", false)

SpeakinSpell:PrintLoading("gui/currentmessages.lua")

-------------------------------------------------------------------------------
-- GUI LAYOUT - Message Settings
-------------------------------------------------------------------------------


SpeakinSpell.OptionsGUI.args.CurrentMessagesGUI = {
	order = 1,
	type = "group",
	name = L["General Settings"],
	desc = L["General Settings TOOLTIP"],
	args = {
		CaptionNormalMode = {
			order = 1,
			type = "description",
			name = L["Message Settings"],
		},
		EventTextFilterSelect = {
			order = 2,
			type = "input",
			width = "full",
			name = L["Search"],
			desc = L["Type some of the name of what you're looking for, to narrow down the list below"],
			get = function() return  SpeakinSpell:CurrentMessagesGUI_OnGetSetEventTextFilter("GET",nil) end,
			set = function(_, value) SpeakinSpell:CurrentMessagesGUI_OnGetSetEventTextFilter("SET",value) end,
		},
		EventTypeFilterSelect = {
			order = 3,
			type = "select",
			width = "full",
			name = L["Select a Category of Events"],
			desc = L["Select a Category of Events TOOLTIP"],
			values = L.EVENTTYPES.AS_FILTERS,
			get = function() return  SpeakinSpell:CurrentMessagesGUI_OnEventTypeFilterSelect("GET",nil) end,
			set = function(_, value) SpeakinSpell:CurrentMessagesGUI_OnEventTypeFilterSelect("SET",value) end,
		},
		ShowMoreThanAHundred = {
			order = 4,
			type = "toggle",
			width = "full",
			name = L["Show More than 100 Search Results"],
			desc = L["Enable to show more than 100 search results in the drop-down list below.\n\nEnabling this option can slow down the performance of this window."],
			get = function() return  SpeakinSpell:CurrentMessagesGUI_OnToggleShowMoreThanAHundred("GET",nil) end,
			set = function(_, value) SpeakinSpell:CurrentMessagesGUI_OnToggleShowMoreThanAHundred("SET",value) end,
		},
		NoSearchResults = {
			name = "\n"..L["No Matching Search Results Found"].."\n",
			order = 5,
			type = "description",
			width = "full",
			hidden = function() return SpeakinSpell:CurrentMessagesGUI_ShowSelectedEventControls() end,
		},
		EventSelect = {
			order = 5,
			type = "select",
			width = "full",
			name = L["Select a Spell"],
			desc = L["Select a Spell TOOLTIP"],
			values = {},
			get = function() return  SpeakinSpell:CurrentMessagesGUI_OnEventSelect("GET",nil) end,
			set = function(_, value) SpeakinSpell:CurrentMessagesGUI_OnEventSelect("SET",value) end,
			hidden = function() return not SpeakinSpell:CurrentMessagesGUI_ShowSelectedEventControls() end,
		},
		CreateSpellButton = {
			order = 6,
			type = "execute",
			name = L["Create New..."],
			desc = L["Create New... TOOLTIP"],
			func = function() SpeakinSpell:CurrentMessagesGUI_OnClickGotoCreate() end,
		},
		DeleteSpellButton = {
			order = 7,
			type = "execute",
			name = L["Delete Spell"],
			desc = L["Delete Spell TOOLTIP"],
			func = function() SpeakinSpell:CurrentMessagesGUI_OnClickDelete() end,
			hidden = function() return not SpeakinSpell:CurrentMessagesGUI_ShowSelectedEventControls() end,
		},
		Disable = {
			order = 8,
			type = "toggle",
			width = "full",
			name = L["Disable announcements for this spell"],
			desc = L["Disable announcements for this spell TOOLTIP"],
			get = function() return  SpeakinSpell:CurrentMessagesGUI_OnSpellDisable("GET",nil) end,
			set = function(_, value) SpeakinSpell:CurrentMessagesGUI_OnSpellDisable("SET",value) end,
			hidden = function() return not SpeakinSpell:CurrentMessagesGUI_ShowSelectedEventControls() end,
		},
		ChannelSelectCaption = {
			order = 10,
			type = "description",
			width= "full",
			name = L["Select the channels to use when announcing this spell when"],
			hidden = function() return not SpeakinSpell:CurrentMessagesGUI_ShowSelectedEventControls() end,
		},
		ChannelSelectSolo = {
			order = 11,
			type = "select",
			name = L["By yourself"],
			desc = L["By yourself TOOLTIP"],
			hidden = function() return not SpeakinSpell:CurrentMessagesGUI_ShowSelectedEventControls() end,
			values = { -- color-coded to match default chat channel colors
				[L["SPEAKINSPELL CHANNEL"]]	= "|cff33ff99"..L["SPEAKINSPELL CHANNEL"],
				[L["GUILD"]]		= "|c4444ff44"..L["GUILD"],
				--[L["BATTLEGROUND"]]	= "|cffff6b00"..L["BATTLEGROUND"],
				--[L["RAID"]]			= "|cffff6b00"..L["RAID"],
				--[L["PARTY"]]		= "|cffaaeeff"..L["PARTY"],
				[L["SAY"]]			=               L["SAY"],
				[L["EMOTE"]]		= "|cffff7b34"..L["EMOTE"],
				[L["YELL"]]			= "|cffff2f32"..L["YELL"],
				--[L["RAID_WARNING"]]	= "|cffffdbad"..L["RAID_WARNING"],
				[L["Silent"]]		=               L["Silent"], --tried light gray to indicate disabled, but it made the control appear to be disabled, so use the default color (white)
			},
			get = function() return  SpeakinSpell:CurrentMessagesGUI_OnChannelSelect("GET","Solo",nil) end,
			set = function(_, value) SpeakinSpell:CurrentMessagesGUI_OnChannelSelect("SET","Solo",value) end,
		},
		ChannelSelectParty = {
			order = 12,
			type = "select",
			name = L["In a Party"],
			desc = L["In a Party TOOLTIP"],
			hidden = function() return not SpeakinSpell:CurrentMessagesGUI_ShowSelectedEventControls() end,
			values = { -- color-coded to match default chat channel colors
				--[L["BATTLEGROUND"]]	= "|cffff6b00"..L["BATTLEGROUND"],
				--[L["RAID"]]			= "|cffff6b00"..L["RAID"],
				[L["SPEAKINSPELL CHANNEL"]]	= "|cff33ff99"..L["SPEAKINSPELL CHANNEL"],
				[L["GUILD"]]		= "|c4444ff44"..L["GUILD"],
				[L["PARTY"]]		= "|cffaaeeff"..L["PARTY"],
				[L["SAY"]]			=               L["SAY"],
				[L["EMOTE"]]		= "|cffff7b34"..L["EMOTE"],
				[L["YELL"]]			= "|cffff2f32"..L["YELL"],
				[L["RAID_WARNING"]]	= "|cffffdbad"..L["RAID_WARNING"],
				[L["Silent"]]		=               L["Silent"], --tried light gray to indicate disabled, but it made the control appear to be disabled, so use the default color (white)
			},
			get = function() return  SpeakinSpell:CurrentMessagesGUI_OnChannelSelect("GET","Party",nil) end,
			set = function(_, value) SpeakinSpell:CurrentMessagesGUI_OnChannelSelect("SET","Party",value) end,
		},
		ChannelSelectRaid = {
			order = 13,
			type = "select",
			name = L["In a Raid"],
			desc = L["In a Raid TOOLTIP"],
			hidden = function() return not SpeakinSpell:CurrentMessagesGUI_ShowSelectedEventControls() end,
			values = { -- color-coded to match default chat channel colors
				--[L["BATTLEGROUND"]]	= "|cffff6b00"..L["BATTLEGROUND"],
				[L["SPEAKINSPELL CHANNEL"]]	= "|cff33ff99"..L["SPEAKINSPELL CHANNEL"],
				[L["GUILD"]]		= "|c4444ff44"..L["GUILD"],
				[L["RAID"]]			= "|cffff6b00"..L["RAID"],
				[L["PARTY"]]		= "|cffaaeeff"..L["PARTY"],
				[L["SAY"]]			=               L["SAY"],
				[L["EMOTE"]]		= "|cffff7b34"..L["EMOTE"],
				[L["YELL"]]			= "|cffff2f32"..L["YELL"],
				[L["RAID_WARNING"]]	= "|cffffdbad"..L["RAID_WARNING"],
				[L["Silent"]]		=               L["Silent"], --tried light gray to indicate disabled, but it made the control appear to be disabled, so use the default color (white)
			},
			get = function() return  SpeakinSpell:CurrentMessagesGUI_OnChannelSelect("GET","Raid",nil) end,
			set = function(_, value) SpeakinSpell:CurrentMessagesGUI_OnChannelSelect("SET","Raid",value) end,
		},
		ChannelSelectPartyLeader = {
			order = 14,
			type = "select",
			name = L["Party Leader"],
			desc = L["Use this channel if you are the leader of a 5-man party"],
			hidden = function() return not SpeakinSpell:CurrentMessagesGUI_ShowSelectedEventControls() end,
			values = { -- color-coded to match default chat channel colors
				--[L["BATTLEGROUND"]]	= "|cffff6b00"..L["BATTLEGROUND"],
				[L["SPEAKINSPELL CHANNEL"]]	= "|cff33ff99"..L["SPEAKINSPELL CHANNEL"],
				[L["GUILD"]]		= "|c4444ff44"..L["GUILD"],
				[L["RAID"]]			= "|cffff6b00"..L["RAID"],
				[L["PARTY"]]		= "|cffaaeeff"..L["PARTY"],
				[L["SAY"]]			=               L["SAY"],
				[L["EMOTE"]]		= "|cffff7b34"..L["EMOTE"],
				[L["YELL"]]			= "|cffff2f32"..L["YELL"],
				[L["RAID_WARNING"]]	= "|cffffdbad"..L["RAID_WARNING"],
				[L["Silent"]]		=               L["Silent"], --tried light gray to indicate disabled, but it made the control appear to be disabled, so use the default color (white)
			},
			get = function() return  SpeakinSpell:CurrentMessagesGUI_OnChannelSelect("GET","PartyLeader",nil) end,
			set = function(_, value) SpeakinSpell:CurrentMessagesGUI_OnChannelSelect("SET","PartyLeader",value) end,
		},
		ChannelSelectRaidLeader = {
			order = 15,
			type = "select",
			name = L["Raid Leader"],
			desc = L["Use this channel if you are the leader of a raid group"],
			hidden = function() return not SpeakinSpell:CurrentMessagesGUI_ShowSelectedEventControls() end,
			values = { -- color-coded to match default chat channel colors
				--[L["BATTLEGROUND"]]	= "|cffff6b00"..L["BATTLEGROUND"],
				[L["SPEAKINSPELL CHANNEL"]]	= "|cff33ff99"..L["SPEAKINSPELL CHANNEL"],
				[L["GUILD"]]		= "|c4444ff44"..L["GUILD"],
				[L["RAID"]]			= "|cffff6b00"..L["RAID"],
				[L["PARTY"]]		= "|cffaaeeff"..L["PARTY"],
				[L["SAY"]]			=               L["SAY"],
				[L["EMOTE"]]		= "|cffff7b34"..L["EMOTE"],
				[L["YELL"]]			= "|cffff2f32"..L["YELL"],
				[L["RAID_WARNING"]]	= "|cffffdbad"..L["RAID_WARNING"],
				[L["Silent"]]		=               L["Silent"], --tried light gray to indicate disabled, but it made the control appear to be disabled, so use the default color (white)
			},
			get = function() return  SpeakinSpell:CurrentMessagesGUI_OnChannelSelect("GET","RaidLeader",nil) end,
			set = function(_, value) SpeakinSpell:CurrentMessagesGUI_OnChannelSelect("SET","RaidLeader",value) end,
		},
		ChannelSelectRaidOfficer = {
			order = 16,
			type = "select",
			name = L["Raid Officer"],
			desc = L["Use this channel if you are promoted to assist in a raid group"],
			hidden = function() return not SpeakinSpell:CurrentMessagesGUI_ShowSelectedEventControls() end,
			values = { -- color-coded to match default chat channel colors
				--[L["BATTLEGROUND"]]	= "|cffff6b00"..L["BATTLEGROUND"],
				[L["SPEAKINSPELL CHANNEL"]]	= "|cff33ff99"..L["SPEAKINSPELL CHANNEL"],
				[L["GUILD"]]		= "|c4444ff44"..L["GUILD"],
				[L["RAID"]]			= "|cffff6b00"..L["RAID"],
				[L["PARTY"]]		= "|cffaaeeff"..L["PARTY"],
				[L["SAY"]]			=               L["SAY"],
				[L["EMOTE"]]		= "|cffff7b34"..L["EMOTE"],
				[L["YELL"]]			= "|cffff2f32"..L["YELL"],
				[L["RAID_WARNING"]]	= "|cffffdbad"..L["RAID_WARNING"],
				[L["Silent"]]		=               L["Silent"], --tried light gray to indicate disabled, but it made the control appear to be disabled, so use the default color (white)
			},
			get = function() return  SpeakinSpell:CurrentMessagesGUI_OnChannelSelect("GET","RaidOfficer",nil) end,
			set = function(_, value) SpeakinSpell:CurrentMessagesGUI_OnChannelSelect("SET","RaidOfficer",value) end,
		},
		ChannelSelectBG = {
			order = 17,
			type = "select",
			name = L["In a Battleground"],
			desc = L["In a Battleground TOOLTIP"],
			hidden = function() return not SpeakinSpell:CurrentMessagesGUI_ShowSelectedEventControls() end,
			values = { -- color-coded to match default chat channel colors
				[L["SPEAKINSPELL CHANNEL"]]	= "|cff33ff99"..L["SPEAKINSPELL CHANNEL"],
				[L["GUILD"]]		= "|c4444ff44"..L["GUILD"],
				[L["BATTLEGROUND"]]	= "|cffff6b00"..L["BATTLEGROUND"],
				--[L["RAID"]]			= "|cffff6b00"..L["RAID"],
				--[L["PARTY"]]		= "|cffaaeeff"..L["PARTY"],
				[L["SAY"]]			=               L["SAY"],
				[L["EMOTE"]]		= "|cffff7b34"..L["EMOTE"],
				[L["YELL"]]			= "|cffff2f32"..L["YELL"],
				--[L["RAID_WARNING"]]	= "|cffffdbad"..L["RAID_WARNING"],
				[L["Silent"]]		=               L["Silent"], --tried light gray to indicate disabled, but it made the control appear to be disabled, so use the default color (white)
			},
			get = function() return  SpeakinSpell:CurrentMessagesGUI_OnChannelSelect("GET","BG",nil) end,
			set = function(_, value) SpeakinSpell:CurrentMessagesGUI_OnChannelSelect("SET","BG",value) end,
		},
		ChannelSelectArena = {
			order = 18,
			type = "select",
			name = L["In Arena"],
			desc = L["In Arena TOOLTIP"],
			hidden = function() return not SpeakinSpell:CurrentMessagesGUI_ShowSelectedEventControls() end,
			values = { -- color-coded to match default chat channel colors
				[L["SPEAKINSPELL CHANNEL"]]	= "|cff33ff99"..L["SPEAKINSPELL CHANNEL"],
				[L["GUILD"]]		= "|c4444ff44"..L["GUILD"],
				--[L["BATTLEGROUND"]]	= "|cffff6b00"..L["BATTLEGROUND"],
				[L["RAID"]]			= "|cffff6b00"..L["RAID"],
				[L["PARTY"]]		= "|cffaaeeff"..L["PARTY"],
				[L["SAY"]]			=               L["SAY"],
				[L["EMOTE"]]		= "|cffff7b34"..L["EMOTE"],
				[L["YELL"]]			= "|cffff2f32"..L["YELL"],
				[L["RAID_WARNING"]]	= "|cffffdbad"..L["RAID_WARNING"],
				[L["Silent"]]		=               L["Silent"], --tried light gray to indicate disabled, but it made the control appear to be disabled, so use the default color (white)
			},
			get = function() return  SpeakinSpell:CurrentMessagesGUI_OnChannelSelect("GET","Arena",nil) end,
			set = function(_, value) SpeakinSpell:CurrentMessagesGUI_OnChannelSelect("SET","Arena",value) end,
		},
		ChannelSelectWG = {
			order = 19,
			type = "select",
			name = L["In Wintergrasp"],
			desc = L["In Wintergrasp TOOLTIP"],
			hidden = function() return not SpeakinSpell:CurrentMessagesGUI_ShowSelectedEventControls() end,
			values = { -- color-coded to match default chat channel colors
				[L["SPEAKINSPELL CHANNEL"]]	= "|cff33ff99"..L["SPEAKINSPELL CHANNEL"],
				[L["GUILD"]]		= "|c4444ff44"..L["GUILD"],
				--[L["BATTLEGROUND"]]	= "|cffff6b00"..L["BATTLEGROUND"],
				[L["RAID"]]			= "|cffff6b00"..L["RAID"],
				[L["PARTY"]]		= "|cffaaeeff"..L["PARTY"],
				[L["SAY"]]			=               L["SAY"],
				[L["EMOTE"]]		= "|cffff7b34"..L["EMOTE"],
				[L["YELL"]]			= "|cffff2f32"..L["YELL"],
				--[L["RAID_WARNING"]]	= "|cffffdbad"..L["RAID_WARNING"],
				[L["Silent"]]		=               L["Silent"], --tried light gray to indicate disabled, but it made the control appear to be disabled, so use the default color (white)
			},
			get = function() return  SpeakinSpell:CurrentMessagesGUI_OnChannelSelect("GET","WG",nil) end,
			set = function(_, value) SpeakinSpell:CurrentMessagesGUI_OnChannelSelect("SET","WG",value) end,
		},
		WhisperTargetToggle = {
			order = 20,
			type = "toggle",
			width = "full",
			name = L["Whisper the message to your target"],
			desc = L["Whisper the message to your target TOOLTIP"],
			get = function() return  SpeakinSpell:CurrentMessagesGUI_OnWhisperTarget("GET",nil) end,
			set = function(_, value) SpeakinSpell:CurrentMessagesGUI_OnWhisperTarget("SET",value) end,
			hidden = function() return not SpeakinSpell:CurrentMessagesGUI_ShowSelectedEventControls() end,
		},
		FrequencySelect = {
			order = 31,
			type = "range",
			width = "full",
			name = L["How often do you want to announce this spell?"],
			desc = L["How often TOOLTIP"],
			min = 0.01, max = 1.00, step=0.01,
			isPercent = true,
			get = function() return  SpeakinSpell:CurrentMessagesGUI_OnFrequencySelect("GET",nil) end,
			set = function(_, value) SpeakinSpell:CurrentMessagesGUI_OnFrequencySelect("SET",value) end,
			hidden = function() return not SpeakinSpell:CurrentMessagesGUI_ShowSelectedEventControls() end,
		},
		CooldownSlider = {
			order = 32,
			type = "range",
			width = "full",
			name = L["Cooldown on announcements for this spell"] ,
			desc = L["Cooldown TOOLTIP"],
			min =0, max = 600, step=1, -- allowing for 0 seconds up to 10 minutes
			get = function() return  SpeakinSpell:CurrentMessagesGUI_OnCooldownSlider("GET",nil) end,
			set = function(_, value) SpeakinSpell:CurrentMessagesGUI_OnCooldownSlider("SET",value) end,
			hidden = function() return not SpeakinSpell:CurrentMessagesGUI_ShowSelectedEventControls() end,
		},
		OncePerCombatToggle = {
			order = 33,
			type = "toggle",
			width = "double",
			name = L["Limit once per combat"],
			desc = L["Limit once per combat TOOLTIP"],
			get = function() return  SpeakinSpell:CurrentMessagesGUI_OnOncePerCombatToggle("GET",nil) end,
			set = function(_, value) SpeakinSpell:CurrentMessagesGUI_OnOncePerCombatToggle("SET",value) end,
			hidden = function() return not SpeakinSpell:CurrentMessagesGUI_ShowSelectedEventControls() end,
		},
		OncePerTargetToggle = {
			order = 34,
			type = "toggle",
			width = "double",
			name = L["Limit once per target"],
			desc = L["Limit once per target TOOLTIP"],
			get = function() return  SpeakinSpell:CurrentMessagesGUI_OnOncePerTargetToggle("GET",nil) end,
			set = function(_, value) SpeakinSpell:CurrentMessagesGUI_OnOncePerTargetToggle("SET",value) end,
			hidden = function() return not SpeakinSpell:CurrentMessagesGUI_ShowSelectedEventControls() end,
		},
	}, --end args
} --end CurrentMessagesGUI


-------------------------------------------------------------------------------
-- OPTIONS GUI FUNCTIONS - MESSAGE SETTINGS
-------------------------------------------------------------------------------


function SpeakinSpell:CurrentMessagesGUI_GetSelectedEventObject()
	local eventkey = self.RuntimeData.OptionsGUIStates.MessageSettings.SelectedEventKey
	if eventkey then
		return SpeakinSpellSavedData.EventTable[eventkey]
	else
		return nil
	end
end


function SpeakinSpell:CurrentMessagesGUI_ValidateSelectedEvent()
	-- get the selected EventTableEntry
	local EventTableEntry = self:CurrentMessagesGUI_GetSelectedEventObject()
	if EventTableEntry then
		if self:MatchesFilter(EventTableEntry.DetectedEvent,true) then
			-- the current selection is valid, so keep it
			return
		end
		--else, the current selection did not match the filter, so select a default
	end
	
	-- select the first thing in the list, if possible
	for key,EventTableEntry in pairs(SpeakinSpellSavedData.EventTable) do
		if self:MatchesFilter(EventTableEntry.DetectedEvent,true) then
			-- the current selection is valid, so keep it
			self.RuntimeData.OptionsGUIStates.MessageSettings.SelectedEventKey = key
			return
		end
	end
	
	-- our spell list is empty or nothing matches the filter
	self.RuntimeData.OptionsGUIStates.MessageSettings.SelectedEventKey = nil
end



function SpeakinSpell:CurrentMessagesGUI_OnClickDelete()
	local funcname = "CurrentMessagesGUI_OnClickDelete"

	-- get the selected EventTableEntry
	local eventkey = self.RuntimeData.OptionsGUIStates.MessageSettings.SelectedEventKey
	if not eventkey then
		self:ErrorMsg(funcname, "no event selected")
		return
	end
	
	-- delete the current selection from the SavedData table
	self:DeleteSpell(eventkey)
end



function SpeakinSpell:CurrentMessagesGUI_OnToggleShowMoreThanAHundred(getset,value)
	if "GET" == getset then
		-- rebuild the list to match the new filter if its out of date
		self:CurrentMessagesGUI_RebuildSpellList()
		return SpeakinSpellSavedData.ShowMoreThanAHundred
	else -- "SET"
		-- if the filter is changing, then rebuild the spell/event list
		self:SetFilterShowMoreThanAHundred(value)
		-- rebuild the list to match the new filter
		self:CurrentMessagesGUI_RebuildSpellList()
	end
end


function SpeakinSpell:CurrentMessagesGUI_OnClickGotoCreate()
	InterfaceOptionsFrame_OpenToCategory(L["Create New..."])
end


function SpeakinSpell:CurrentMessagesGUI_OnEventTypeFilterSelect(getset, value)
	if "GET" == getset then
		-- rebuild the list to match the new filter if its out of date
		self:CurrentMessagesGUI_RebuildSpellList()
		return SpeakinSpell.RuntimeData.OptionsGUIStates.SelectedEventTypeFilter
	else -- "SET"
		-- if the filter is changing, then rebuild the spell/event list
		self:SetFilter(value,nil)
		-- rebuild the list to match the new filter
		self:CurrentMessagesGUI_RebuildSpellList()
	end
	-- NOTE: the current selection will be repaired to match the new filter in _ValidateSelectedEvent
	--		as a side effect when the GUI framework automatically tries to get the new value of 
	--		the rest of the controls on the page, including the selected event
end


function SpeakinSpell:CurrentMessagesGUI_OnGetSetEventTextFilter(getset, value)
	if "GET" == getset then
		-- rebuild the list to match the new filter if its out of date
		self:CurrentMessagesGUI_RebuildSpellList()
		return SpeakinSpell.RuntimeData.OptionsGUIStates.SelectedEventTextFilter
	else -- "SET"
		-- if the filter is changing, then rebuild the spell/event list
		self:SetFilter(nil,value)
		-- rebuild the list to match the new filter
		self:CurrentMessagesGUI_RebuildSpellList()
	end
	-- NOTE: the current selection will be repaired to match the new filter in _ValidateSelectedEvent
	--		as a side effect when the GUI framework automatically tries to get the new value of 
	--		the rest of the controls on the page, including the selected event
end

function SpeakinSpell:CurrentMessagesGUI_OnEventSelect(getset, value)
	if "GET" == getset then
		-- create a side-effect to update the shown/hidden state of all options GUI controls
		self:CurrentMessagesGUI_RebuildSpellList()
		-- return the current spell selection
		return self.RuntimeData.OptionsGUIStates.MessageSettings.SelectedEventKey
	else -- "SET"
		-- update the current selection
		self.RuntimeData.OptionsGUIStates.MessageSettings.SelectedEventKey = value
	end
end


function SpeakinSpell:CurrentMessagesGUI_OnChannelSelect(getset, scenario, value)
	local funcname = "CurrentMessagesGUI_OnChannelSelect"
		
	-- get the selected EventTableEntry
	local EventTableEntry = self:CurrentMessagesGUI_GetSelectedEventObject()
	if not EventTableEntry then
		self:ErrorMsg(funcname, "no event selected")
		return
	end

	if "GET" == getset then
		--self:DebugMsg(funcname,"GET "..scenario..":"..tostring(EventTableEntry.Channels[scenario]))
		if EventTableEntry.Channels[scenario] then
			return L[EventTableEntry.Channels[scenario]]
		else
			return L["Silent"]
		end
	else --"SET"
		-- store the new channel selection
		if value == L["Silent"] then
			EventTableEntry.Channels[scenario] = nil
		else
			-- create a reverse lookup table to un-localize channel names
			-- so we can return a channel name that is compatible with SendChatMessage()
			local CHANNEL_TABLE = {
				[L["SAY"]]			= "SAY",
				[L["PARTY"]]		= "PARTY",
				[L["RAID"]]			= "RAID",
				[L["EMOTE"]]		= "EMOTE",
				[L["BATTLEGROUND"]]	= "BATTLEGROUND",
				[L["YELL"]]			= "YELL",
				[L["RAID_WARNING"]]	= "RAID_WARNING",
				[L["GUILD"]]		= "GUILD",
				[L["SPEAKINSPELL CHANNEL"]] = "SPEAKINSPELL CHANNEL",
			}
			EventTableEntry.Channels[scenario] = CHANNEL_TABLE[value]
		end
		--self:DebugMsg(funcname,"set "..scenario..":"..tostring(EventTableEntry.Channels[scenario]))
	end
end



function SpeakinSpell:CurrentMessagesGUI_OnWhisperTarget(getset, value)
	-- make sure we have a valid spell selection
	local EventTableEntry = self:CurrentMessagesGUI_GetSelectedEventObject()
	if not EventTableEntry then
		self:ErrorMsg( "CurrentMessagesGUI_OnWhisperTarget", "no event selected" )
		return false
	end
	
	if "GET" == getset then
		return EventTableEntry.WhisperTarget
	else
		EventTableEntry.WhisperTarget = value
	end
end


function SpeakinSpell:CurrentMessagesGUI_OnMessage(getset, index, value)
	local EventTableEntry = self:CurrentMessagesGUI_GetSelectedEventObject()
	if not EventTableEntry then
		self:ErrorMsg("CurrentMessagesGUI_OnMessage", "no event selected")
		return ""
	end
	
	if "GET" == getset then
		local msg = EventTableEntry.Messages[index]
		if not msg then
			msg = ""
		end
		return msg
	else -- "SET"
		-- make sure we have a message table for the currently selected spell
		if not EventTableEntry.Messages then
			EventTableEntry.Messages = {}
		end
		
		-- store the new value into the message table
		EventTableEntry.Messages[index] = value
		-- self:DebugMsg( nil,"Message " .. tostring(index) .. " for spell " .. EventTableEntry.DetectedEvent.DisplayName .. " set to: " .. tostring(EventTableEntry.Messages[index]) )
		
		-- remove holes from the message table
		-- if the user has just deleted one of the entries, move everything else up to fill in the gap
		if value == "" then
			local Messages = EventTableEntry.Messages -- create reference for shortcut notation
			for i=index+1,SpeakinSpell.MAX.MESSAGES_PER_SPELL,1 do
				Messages[i-1] = Messages[i]
			end
			Messages[SpeakinSpell.MAX.MESSAGES_PER_SPELL] = nil
			-- destroy empty strings from the table
			for i=SpeakinSpell.MAX.MESSAGES_PER_SPELL-1,1,-1 do
				if Messages[i] == "" then
					Messages[i] = nil
				else
					break --the rest should be valid non-empty strings
				end
			end
		end
	end
end




function SpeakinSpell:CurrentMessagesGUI_OnSpellDisable(getset, value)
	local EventTableEntry = self:CurrentMessagesGUI_GetSelectedEventObject()
	if not EventTableEntry then
		self:ErrorMsg("CurrentMessagesGUI_OnSpellDisable", "no event selected")
		return
	end

	if "GET" == getset then
		return EventTableEntry.DisableAnnouncements
	else -- "SET"
		EventTableEntry.DisableAnnouncements = value
	end
end


function SpeakinSpell:CurrentMessagesGUI_OnFrequencySelect(getset, value)
	local EventTableEntry = self:CurrentMessagesGUI_GetSelectedEventObject()
	if not EventTableEntry then
		self:ErrorMsg("CurrentMessagesGUI_OnFrequencySelect", "no event selected")
		return
	end
	
	if "GET" == getset then	
		return EventTableEntry.Frequency
	else --"SET"
		EventTableEntry.Frequency = value
	end
end


function SpeakinSpell:CurrentMessagesGUI_OnCooldownSlider(getset, value)
	local funcname = "CurrentMessagesGUI_OnCooldownSlider"

	local EventTableEntry = self:CurrentMessagesGUI_GetSelectedEventObject()
	if not EventTableEntry then
		self:ErrorMsg(funcname, "no event selected")
		return
	end
	
	if "GET" == getset then	
		return EventTableEntry.Cooldown
	else --"SET"
		EventTableEntry.Cooldown = value
	end
end


function SpeakinSpell:CurrentMessagesGUI_OnOncePerCombatToggle(getset, value)
	local funcname = "CurrentMessagesGUI_OnOncePerCombatToggle"

	local EventTableEntry = self:CurrentMessagesGUI_GetSelectedEventObject()
	if not EventTableEntry then
		self:ErrorMsg(funcname, "no event selected")
		return
	end
	
	if "GET" == getset then	
		return EventTableEntry.OncePerCombat
	else --"SET"
		EventTableEntry.OncePerCombat = value
	end
end


function SpeakinSpell:CurrentMessagesGUI_OnOncePerTargetToggle(getset, value)
	local funcname = "CurrentMessagesGUI_OnOncePerTargetToggle"

	local EventTableEntry = self:CurrentMessagesGUI_GetSelectedEventObject()
	if not EventTableEntry then
		self:ErrorMsg(funcname, "no event selected")
		return
	end
	
	if "GET" == getset then	
		return EventTableEntry.OncePerTarget
	else --"SET"
		EventTableEntry.OncePerTarget = value
	end
end


function SpeakinSpell:CurrentMessagesGUI_ShowSpeechBox(i)	
	-- determine how many Spell Message N input boxes are shown
	-- typically we'll show at least 1 speech message input box if we have a spell selected
	-- we want to show 1 more than the number of speeches currently configured
	-- unless we don't have a spell selected or we're in EnterNewSpell mode
	local NumSpellMessageInputsShown = 0
	local EventTableEntry = self:CurrentMessagesGUI_GetSelectedEventObject()
	if EventTableEntry then
		NumSpellMessageInputsShown = 1 + #(EventTableEntry.Messages)
	end

	return (i <= NumSpellMessageInputsShown)
end


function SpeakinSpell:CurrentMessagesGUI_ShowSelectedEventControls()
	local EventTableEntry = self:CurrentMessagesGUI_GetSelectedEventObject()
	return (EventTableEntry ~= nil)
end


function SpeakinSpell:CurrentMessagesGUI_RebuildSpellList()
	local values = self.OptionsGUI.args.CurrentMessagesGUI.args.EventSelect.values
	local OptionsGUIState = self.RuntimeData.OptionsGUIStates.MessageSettings
	local EventTable = SpeakinSpellSavedData.EventTable
	local MatchesFilterFunc = function(key) 
		local de = SpeakinSpellSavedData.EventTable[key].DetectedEvent
		return SpeakinSpell:MatchesFilter( de, true )
	end
	local GetDisplayNameFunc = function(key)
		local de = SpeakinSpellSavedData.EventTable[key].DetectedEvent
		return de.displayname
	end
	self:RebuildSpellList( values, OptionsGUIState, EventTable, MatchesFilterFunc, GetDisplayNameFunc )
end


-- create message boxes
function SpeakinSpell:CurrentMessagesGUI_CreateGUI()
	-- add SpeakinSpell.MAX.MESSAGES_PER_SPELL random speech input boxes
	for i=1,SpeakinSpell.MAX.MESSAGES_PER_SPELL,1 do
		self.OptionsGUI.args.CurrentMessagesGUI.args["SpellMessage"..i] = {
			order = i + 100, -- significantly higher order than the rest of the window
			width = "full",
			type = "input",
			name = string.format(L["Random Speech i"], i), -- "Random Speech %d"
			desc = L["Random Speech i TOOLTIP"],
			get = function() return  self:CurrentMessagesGUI_OnMessage("GET",i,nil) end,
			set = function(_, value) self:CurrentMessagesGUI_OnMessage("SET",i, value) end,
			hidden = function() return not self:CurrentMessagesGUI_ShowSpeechBox(i) end,
		}
	end
end
